==> mlDHT_1.0.0_src.zip: lbms/plugins/mldht/kad/messages/GetPeersResponse.java <==
package lbms.plugins.mldht.kad.messages;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import lbms.plugins.mldht.kad.DBItem;
import lbms.plugins.mldht.kad.DHT;
import lbms.plugins.mldht.kad.DHTConstants;
import lbms.plugins.mldht.kad.Key;
import lbms.plugins.mldht.kad.PackUtil;

import org.gudy.azureus2.core3.util.BEncoder;

/**
 * @author Damokles
 *
 */
public class GetPeersResponse extends MessageBase {

	private Key				token;
	private byte[]			data;

	private List<DBItem>	items;

	/**
	 * @param mtid
	 * @param id
	 * @param data
	 * @param token
	 */
	public GetPeersResponse (byte mtid, Key id, byte[] data, Key token) {
		super(mtid, Method.GET_PEERS, Type.RSP_MSG, id);
		this.data = data;
		this.token = token;
	}

	/**
	 * @param mtid
	 * @param id
	 * @param items
	 * @param token
	 */
	public GetPeersResponse (byte mtid, Key id, List<DBItem> items, Key token) {
		super(mtid, Method.GET_PEERS, Type.RSP_MSG, id);
		this.items = items;
		this.token = token;
	}

	/* (non-Javadoc)
	 * @see lbms.plugins.mldht.kad.messages.MessageBase#apply(lbms.plugins.mldht.kad.DHT)
	 */
	@Override
	public void apply (DHT dh_table) {
		dh_table.response(this);
	}

	/* (non-Javadoc)
	 * @see lbms.plugins.mldht.kad.messages.MessageBase#encode(byte[])
	 */
	@Override
	public byte[] encode () throws IOException {
		//BEncoder enc(new BEncoderBufferOutput(arr));
		Map<String, Object> base = new HashMap<String, Object>();
		Map<String, Object> innerMap = new HashMap<String, Object>();
		base.put(DHTConstants.RSP, innerMap);
		innerMap.put("id", id.getHash());
		innerMap.put("token", token.getHash());
		if (data != null && data.length > 0) {
			innerMap.put("nodes", data);
		} else {
			List<byte[]> itemsList = new ArrayList<byte[]>(1);
			itemsList.add(PackUtil.PackDBItems(items));
			innerMap.put("values", itemsList);
		}

		base.put(DHTConstants.TID, new String(new byte[] { mtid }, "UTF-8"));
		base.put(DHTConstants.TYP, DHTConstants.RSP);

		return BEncoder.encode(base);
	}

	//GetPeersRsp(byte mtid,Key id, DBItemList  values,Key  token);

	public byte[] getData () {
		return data;
	}

	public List<DBItem> getItemList () {
		return Collections.unmodifiableList(items);
	}

	public Key getToken () {
		return token;
	}

	public boolean containsNodes () {
		return data != null && data.length > 0;
	}

	public boolean containsValues () {
		return data == null || data.length == 0;
	}
}

==> mlDHT_1.2.6_src.zip: lbms/plugins/mldht/kad/messages/GetPeersResponse.java <==
package lbms.plugins.mldht.kad.messages;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import lbms.plugins.mldht.kad.DBItem;
import lbms.plugins.mldht.kad.DHT;
import lbms.plugins.mldht.kad.DHTConstants;
import lbms.plugins.mldht.kad.Key;

import org.gudy.azureus2.core3.util.BEncoder;

/**
 * @author Damokles
 *
 */
public class GetPeersResponse extends MessageBase {

	private Key				token;
	private byte[]			data;

	private List<DBItem>	items;

	/**
	 * @param mtid
	 * @param id
	 * @param data
	 * @param token
	 */
	public GetPeersResponse (byte[] mtid, Key id, byte[] data, Key token) {
		super(mtid, Method.GET_PEERS, Type.RSP_MSG, id);
		this.data = data;
		this.token = token;
	}

	/**
	 * @param mtid
	 * @param id
	 * @param items
	 * @param token
	 */
	public GetPeersResponse (byte[] mtid, Key id, List<DBItem> items, Key token) {
		super(mtid, Method.GET_PEERS, Type.RSP_MSG, id);
		this.items = items;
		this.token = token;
	}

	/* (non-Javadoc)
	 * @see lbms.plugins.mldht.kad.messages.MessageBase#apply(lbms.plugins.mldht.kad.DHT)
	 */
	@Override
	public void apply (DHT dh_table) {
		dh_table.response(this);
	}

	/* (non-Javadoc)
	 * @see lbms.plugins.mldht.kad.messages.MessageBase#encode(byte[])
	 */
	@Override
	public byte[] encode () throws IOException {
		//BEncoder enc(new BEncoderBufferOutput(arr));
		Map<String, Object> base = new HashMap<String, Object>();
		Map<String, Object> innerMap = new HashMap<String, Object>();
		base.put(DHTConstants.RSP, innerMap);
		innerMap.put("id", id.getHash());
		innerMap.put("token", token.getHash());
		if (data != null && data.length > 0) {
			innerMap.put("nodes", data);
		} else {
			List<byte[]> itemsList = new ArrayList<byte[]>(items.size());
			for (DBItem item : items) {
				itemsList.add(item.getData());
			}
			innerMap.put("values", itemsList);
		}

		base.put(DHTConstants.TID, mtid);
		base.put(DHTConstants.TYP, DHTConstants.RSP);
		base.put(DHTConstants.VER, DHTConstants.getVersion());

		return BEncoder.encode(base);
	}

	//GetPeersRsp(byte mtid,Key id, DBItemList  values,Key  token);

	public byte[] getData () {
		return data;
	}

	public List<DBItem> getItemList () {
		return Collections.unmodifiableList(items);
	}

	public Key getToken () {
		return token;
	}

	public boolean containsNodes () {
		return data != null && data.length > 0;
	}

	public boolean containsValues () {
		return data == null || data.length == 0;
	}
	
	public String toString() {
		return super.toString() + "contains: "+ (containsNodes() ? (data.length/26)+" nodes" : items.size()+" values");
	}
}
